@@ -7,6 +7,7 @@ from typing import (Optional,
                     TypeVar)
 
 from ground.hints import Point
+from reprit import seekers
 from reprit.base import generate_repr
 
 from .enums import OverlapKind
@@ -61,8 +62,9 @@ class RightEvent(Event):
 
 class LeftNaryEvent(LeftEvent):
     @classmethod
-    def from_segment_endpoints(cls, segment_endpoints: SegmentEndpoints
-                               ) -> 'LeftNaryEvent':
+    def from_segment_endpoints(
+            cls, segment_endpoints: SegmentEndpoints
+    ) -> 'LeftNaryEvent':
         start, end = segment_endpoints
         if start > end:
             start, end = end, start
@@ -102,27 +104,29 @@ class LeftBinaryEvent(LeftEvent):
     @classmethod
     def from_segment_endpoints(cls,
                                segment_endpoints: SegmentEndpoints,
-                               from_first: bool) -> 'LeftBinaryEvent':
+                               from_first_operand: bool) -> 'LeftBinaryEvent':
         start, end = segment_endpoints
         if start > end:
             start, end = end, start
-        event = cls(start, None, from_first)
+        event = cls(start, None, from_first_operand)
         event.right = RightBinaryEvent(end, event)
         return event
 
-    __slots__ = 'from_first', 'start'
+    __slots__ = 'from_first_operand', 'start'
 
     def __init__(self,
                  start: Point,
                  right: Optional['RightBinaryEvent'],
-                 from_first: bool) -> None:
-        self.from_first, self.right, self.start = from_first, right, start
+                 from_first_operand: bool) -> None:
+        self.from_first_operand, self.right, self.start = (
+            from_first_operand, right, start
+        )
 
     __repr__ = recursive_repr()(generate_repr(__init__))
 
     def divide(self, point: Point) -> 'LeftBinaryEvent':
         tail = self.right.left = LeftBinaryEvent(point, self.right,
-                                                 self.from_first)
+                                                 self.from_first_operand)
         self.right = RightBinaryEvent(point, self)
         return tail
 
@@ -136,39 +140,40 @@ class RightBinaryEvent(RightEvent):
     __repr__ = recursive_repr()(generate_repr(__init__))
 
     @property
-    def from_first(self) -> bool:
-        return self.left.from_first
+    def from_first_operand(self) -> bool:
+        return self.left.from_first_operand
 
 
 class LeftMixedEvent(LeftEvent):
     @classmethod
     def from_endpoints(cls,
                        segment_endpoints: SegmentEndpoints,
-                       from_first: bool) -> 'LeftMixedEvent':
+                       from_first_operand: bool) -> 'LeftMixedEvent':
         start, end = segment_endpoints
         inside_on_left = True
         if start > end:
             start, end = end, start
             inside_on_left = False
-        result = cls(start, None, from_first, inside_on_left)
+        result = cls(start, None, from_first_operand, inside_on_left)
         result.right = RightMixedEvent(end, result)
         return result
 
-    __slots__ = ('from_first', 'from_result', 'interior_to_left', 'is_overlap',
-                 'other_interior_to_left', 'start')
+    __slots__ = ('from_first_operand', 'from_result', 'interior_to_left',
+                 'is_overlap', 'other_interior_to_left', 'start')
 
     def __init__(self,
                  start: Point,
                  right: Optional['RightMixedEvent'],
-                 from_first: bool,
+                 from_first_operand: bool,
                  interior_to_left: bool,
                  other_interior_to_left: bool = False,
                  is_overlap: bool = False,
                  from_result: bool = False) -> None:
         self.right, self.start = right, start
-        self.from_first = from_first
+        self.from_first_operand = from_first_operand
         self.interior_to_left, self.other_interior_to_left = (
-            interior_to_left, other_interior_to_left)
+            interior_to_left, other_interior_to_left
+        )
         self.is_overlap = is_overlap
         self.from_result = from_result
 
@@ -187,7 +192,9 @@ class LeftMixedEvent(LeftEvent):
 
     def divide(self, point: Point) -> 'LeftMixedEvent':
         tail = self.right.left = LeftMixedEvent(
-                point, self.right, self.from_first, self.interior_to_left)
+                point, self.right, self.from_first_operand,
+                self.interior_to_left
+        )
         self.right = RightMixedEvent(point, self)
         return tail
 
@@ -201,8 +208,8 @@ class RightMixedEvent(RightEvent):
     __repr__ = recursive_repr()(generate_repr(__init__))
 
     @property
-    def from_first(self) -> bool:
-        return self.left.from_first
+    def from_first_operand(self) -> bool:
+        return self.left.from_first_operand
 
     @property
     def primary(self) -> LeftMixedEvent:
@@ -213,38 +220,41 @@ class LeftHolelessEvent(LeftEvent):
     @classmethod
     def from_endpoints(cls,
                        segment_endpoints: SegmentEndpoints,
-                       from_first: bool) -> 'LeftHolelessEvent':
+                       from_first_operand: bool) -> 'LeftHolelessEvent':
         start, end = segment_endpoints
         inside_on_left = True
         if start > end:
             start, end = end, start
             inside_on_left = False
-        event = cls(start, None, from_first, inside_on_left)
+        event = cls(start, None, from_first_operand, inside_on_left)
         event.right = RightShapedEvent(end, event)
         return event
 
-    __slots__ = ('from_first', 'from_shaped_result', 'interior_to_left',
-                 'other_interior_to_left', 'overlap_kind', 'position',
+    __slots__ = ('from_first_operand', 'from_shaped_result', 'id',
+                 'interior_to_left', 'other_interior_to_left', 'overlap_kind',
                  'start')
 
     def __init__(self,
                  start: Point,
                  right: Optional['RightShapedEvent'],
-                 from_first: bool,
+                 from_first_operand: bool,
                  interior_to_left: bool,
-                 other_interior_to_left: bool = False,
-                 overlap_kind: OverlapKind = OverlapKind.NONE,
+                 *,
                  from_shaped_result: bool = False,
-                 position: int = 0) -> None:
-        self.right, self.start = right, start
-        self.from_first = from_first
-        self.interior_to_left = interior_to_left
-        self.other_interior_to_left = other_interior_to_left
-        self.overlap_kind = overlap_kind
-        self.from_shaped_result = from_shaped_result
-        self.position = position
+                 id_: int = 0,
+                 other_interior_to_left: bool = False,
+                 overlap_kind: OverlapKind = OverlapKind.NONE) -> None:
+        (
+            self.from_first_operand, self.from_shaped_result, self.id,
+            self.interior_to_left, self.other_interior_to_left,
+            self.overlap_kind, self.right, self.start
+        ) = (
+            from_first_operand, from_shaped_result, id_, interior_to_left,
+            other_interior_to_left, overlap_kind, right, start
+        )
 
-    __repr__ = recursive_repr()(generate_repr(__init__))
+    __repr__ = recursive_repr()(generate_repr(__init__,
+                                              field_seeker=seekers.complex_))
 
     @property
     def inside(self) -> bool:
@@ -297,7 +307,8 @@ class LeftHolelessEvent(LeftEvent):
 
     def divide(self, point: Point) -> 'LeftHolelessEvent':
         tail = self.right.left = LeftHolelessEvent(
-                point, self.right, self.from_first, self.interior_to_left
+                point, self.right, self.from_first_operand,
+                self.interior_to_left
         )
         self.right = RightShapedEvent(point, self)
         return tail
@@ -307,52 +318,55 @@ class LeftHoleyEvent(LeftEvent):
     @classmethod
     def from_endpoints(cls,
                        segment_endpoints: SegmentEndpoints,
-                       from_first: bool) -> 'LeftHoleyEvent':
+                       from_first_operand: bool) -> 'LeftHoleyEvent':
         start, end = segment_endpoints
         interior_to_left = True
         if start > end:
             start, end = end, start
             interior_to_left = False
-        event = cls(start, None, from_first, interior_to_left)
+        event = cls(start, None, from_first_operand, interior_to_left)
         event.right = RightShapedEvent(end, event)
         return event
 
-    __slots__ = ('below_event_from_shaped_result', 'contour_id', 'from_first',
-                 'from_in_to_out', 'from_shaped_result', 'interior_to_left',
-                 'other_interior_to_left', 'overlap_kind', 'position',
-                 'start', 'start_index')
+    __slots__ = ('below_event_from_shaped_result', 'contour_id',
+                 'from_first_operand', 'from_in_to_out', 'from_shaped_result',
+                 'interior_to_left', 'other_interior_to_left', 'overlap_kind',
+                 'id', 'start', 'start_id')
 
-    def __init__(self,
-                 start: Point,
-                 right: Optional['RightShapedEvent'],
-                 from_first: bool,
-                 interior_to_left: bool,
-                 other_interior_to_left: bool = False,
-                 overlap_kind: OverlapKind = OverlapKind.NONE,
-                 from_shaped_result: bool = False,
-                 from_in_to_out: bool = False,
-                 position: int = UNDEFINED_INDEX,
-                 start_index: int = UNDEFINED_INDEX,
-                 contour_id: Optional[int] = None,
-                 below_event_from_shaped_result: Optional['LeftHoleyEvent']
-                 = None) -> None:
-        self.right, self.start = right, start
-        self.from_first = from_first
-        self.from_shaped_result = from_shaped_result
-        self.interior_to_left = interior_to_left
-        self.other_interior_to_left = other_interior_to_left
-        self.overlap_kind = overlap_kind
-        self.position = position
-        self.start_index = start_index
-        self.from_in_to_out = from_in_to_out
-        self.contour_id = contour_id
-        self.below_event_from_shaped_result = below_event_from_shaped_result
+    def __init__(
+            self,
+            start: Point,
+            right: Optional['RightShapedEvent'],
+            from_first_operand: bool,
+            interior_to_left: bool,
+            *,
+            below_event_from_shaped_result: Optional['LeftHoleyEvent'] = None,
+            contour_id: Optional[int] = None,
+            from_in_to_out: bool = False,
+            from_shaped_result: bool = False,
+            id_: int = UNDEFINED_INDEX,
+            other_interior_to_left: bool = False,
+            overlap_kind: OverlapKind = OverlapKind.NONE,
+            start_id: int = UNDEFINED_INDEX
+    ) -> None:
+        (
+            self.below_event_from_shaped_result, self.contour_id,
+            self.from_first_operand, self.from_in_to_out,
+            self.from_shaped_result, self.id, self.interior_to_left,
+            self.other_interior_to_left, self.overlap_kind, self.right,
+            self.start, self.start_id
+        ) = (
+            below_event_from_shaped_result, contour_id, from_first_operand,
+            from_in_to_out, from_shaped_result, id_, interior_to_left,
+            other_interior_to_left, overlap_kind, right, start, start_id
+        )
 
-    __repr__ = recursive_repr()(generate_repr(__init__))
+    __repr__ = recursive_repr()(generate_repr(__init__,
+                                              field_seeker=seekers.complex_))
 
     @property
-    def end_index(self) -> int:
-        return self.opposite.start_index
+    def end_id(self) -> int:
+        return self.opposite.start_id
 
     @property
     def inside(self) -> bool:
@@ -409,7 +423,8 @@ class LeftHoleyEvent(LeftEvent):
 
     def divide(self, point: Point) -> 'LeftHoleyEvent':
         tail = self.right.left = LeftHoleyEvent(
-                point, self.right, self.from_first, self.interior_to_left
+                point, self.right, self.from_first_operand,
+                self.interior_to_left
         )
         self.right = RightShapedEvent(point, self)
         return tail
@@ -419,26 +434,28 @@ LeftShapedEvent = TypeVar('LeftShapedEvent', LeftHolelessEvent, LeftHoleyEvent)
 
 
 class RightShapedEvent(RightEvent):
-    __slots__ = 'position', 'start', 'start_index'
+    __slots__ = 'id', 'start', 'start_id'
 
     def __init__(self,
                  start: Point,
                  left: LeftShapedEvent,
-                 position: int = UNDEFINED_INDEX,
-                 start_index: int = UNDEFINED_INDEX) -> None:
-        self.left, self.position, self.start, self.start_index = (
-            left, position, start, start_index
+                 *,
+                 id_: int = UNDEFINED_INDEX,
+                 start_id: int = UNDEFINED_INDEX) -> None:
+        self.id, self.left, self.start, self.start_id = (
+            id_, left, start, start_id
         )
 
-    __repr__ = recursive_repr()(generate_repr(__init__))
+    __repr__ = recursive_repr()(generate_repr(__init__,
+                                              field_seeker=seekers.complex_))
 
     @property
-    def end_index(self) -> int:
-        return self.opposite.start_index
+    def end_id(self) -> int:
+        return self.opposite.start_id
 
     @property
-    def from_first(self) -> bool:
-        return self.left.from_first
+    def from_first_operand(self) -> bool:
+        return self.left.from_first_operand
 
     @property
     def primary(self) -> LeftShapedEvent:
@@ -456,20 +473,19 @@ def events_to_connectivity(events: Sequence[Event]) -> Sequence[int]:
                and events[index].start == current_start
                and not events[index].is_left):
             index += 1
-        right_stop_index = index - 1
         left_start_index = index
         while index < events_count and events[index].start == current_start:
             index += 1
         left_stop_index = index - 1
-        has_right_events = right_stop_index >= right_start_index
+        has_right_events = left_start_index >= right_start_index + 1
         has_left_events = left_stop_index >= left_start_index
         if has_right_events:
-            result[right_start_index:right_stop_index] = range(
-                    right_start_index + 1, right_stop_index + 1
+            result[right_start_index:left_start_index - 1] = range(
+                    right_start_index + 1, left_start_index - 1 + 1
             )
-            result[right_stop_index] = (left_stop_index
-                                        if has_left_events
-                                        else right_start_index)
+            result[left_start_index - 1] = (left_stop_index
+                                            if has_left_events
+                                            else right_start_index)
         if has_left_events:
             result[left_start_index] = (right_start_index
                                         if has_right_events
