@@ -72,45 +72,49 @@ class Operation(ABC):
     def compute_fields(self, event: LeftEvent, below_event: Optional[LeftEvent]
                        ) -> None:
         if below_event is not None:
-            event.other_interior_to_left = (below_event.other_interior_to_left
-                                            if (event.from_first
-                                                is below_event.from_first)
-                                            else below_event.interior_to_left)
+            event.other_interior_to_left = (
+                below_event.other_interior_to_left
+                if event.from_first_operand is below_event.from_first_operand
+                else below_event.interior_to_left
+            )
             event.below_event_from_shaped_result = (
                 below_event.below_event_from_shaped_result
                 if (not self.from_shaped_result(below_event)
                     or below_event.is_vertical)
-                else below_event)
+                else below_event
+            )
         event.from_shaped_result = self.from_shaped_result(event)
 
     def events_to_polygons(self, events: Sequence[Event]) -> Sequence[Polygon]:
-        if not events:
-            return []
-        max_start_index = events[-1].start_index
-        assert max_start_index != UNDEFINED_INDEX
-        assert all(event.start_index <= max_start_index for event in events)
         events = [event
                   for event in events
                   if event.primary.from_shaped_result]
+        if not events:
+            return []
+        max_endpoint_id = events[-1].start_id
+        assert max_endpoint_id != UNDEFINED_INDEX
+        assert all(event.start_id <= max_endpoint_id for event in events)
         events.sort(key=self._events_queue.key)
-        for index, event in enumerate(events):
-            event.position = index
+        for event_id, event in enumerate(events):
+            event.id = event_id
         are_internal, depths, holes, parents = [], [], [], []
-        processed = [False] * len(events)
+        are_events_processed = [False] * len(events)
         context = self.context
         contour_cls, orienteer = context.contour_cls, context.angle_orientation
         contours = []  # type: List[Contour]
         connectivity = events_to_connectivity(events)
-        visited_positions = [UNDEFINED_INDEX] * (max_start_index + 1)
-        for index, event in enumerate(events):
-            if processed[index]:
+        visited_endpoints_positions = [UNDEFINED_INDEX] * (max_endpoint_id + 1)
+        for event_id, event in enumerate(events):
+            if are_events_processed[event_id]:
                 continue
             contour_id = len(contours)
             _compute_relations(event, contour_id, are_internal, depths, holes,
                                parents)
             contour_events = _to_contour_events(event, events, connectivity,
-                                                processed, visited_positions)
-            _process_contour_events(contour_events, contour_id, processed)
+                                                are_events_processed,
+                                                visited_endpoints_positions)
+            _process_contour_events(contour_events, contour_id,
+                                    are_events_processed)
             vertices = _contour_events_to_vertices(contour_events, orienteer)
             if depths[contour_id] % 2:
                 # holes will be in clockwise order
@@ -118,19 +122,17 @@ class Operation(ABC):
             contours.append(contour_cls(vertices))
         result = []
         polygon_cls = context.polygon_cls
-        for index, contour in enumerate(contours):
-            if are_internal[index]:
+        for contour_id, contour in enumerate(contours):
+            if are_internal[contour_id]:
                 # hole of a hole is an external polygon
-                result.extend(
-                        polygon_cls(contours[hole_index],
-                                    [contours[hole_hole_index]
-                                     for hole_hole_index in holes[hole_index]])
-                        for hole_index in holes[index]
-                )
+                result.extend(polygon_cls(contours[hole_id],
+                                          [contours[hole_hole_id]
+                                           for hole_hole_id in holes[hole_id]])
+                              for hole_id in holes[contour_id])
             else:
                 result.append(polygon_cls(contour,
-                                          [contours[hole_index]
-                                           for hole_index in holes[index]]))
+                                          [contours[hole_id]
+                                           for hole_id in holes[contour_id]]))
         return result
 
     def fill_queue(self) -> None:
@@ -155,17 +157,16 @@ class Operation(ABC):
                       processed_events: List[Event],
                       sweep_line: SweepLine) -> None:
         if not event.is_left:
-            processed_events.append(event)
-            event = event.left
-            if event in sweep_line:
-                above_event, below_event = (sweep_line.above(event),
-                                            sweep_line.below(event))
-                sweep_line.remove(event)
+            opposite_event = event.left
+            if opposite_event in sweep_line:
+                above_event, below_event = (sweep_line.above(opposite_event),
+                                            sweep_line.below(opposite_event))
+                sweep_line.remove(opposite_event)
                 if above_event is not None and below_event is not None:
                     self._events_queue.detect_intersection(below_event,
                                                            above_event)
-        elif event not in sweep_line:
             processed_events.append(event)
+        elif event not in sweep_line:
             sweep_line.add(event)
             above_event, below_event = (sweep_line.above(event),
                                         sweep_line.below(event))
@@ -181,6 +182,7 @@ class Operation(ABC):
                 below_below_event = sweep_line.below(below_event)
                 self.compute_fields(below_event, below_below_event)
                 self.compute_fields(event, below_event)
+            processed_events.append(event)
 
     def sweep(self) -> List[Event]:
         self.fill_queue()
@@ -188,15 +190,15 @@ class Operation(ABC):
         sweep_line = SweepLine(self.context)
         events_queue = self._events_queue
         event = events_queue.pop()
-        start = event.start
-        start_index = event.start_index = 0
+        current_endpoint = event.start
+        current_endpoint_id = event.start_id = 0
         self.process_event(event, result, sweep_line)
         while events_queue:
             event = events_queue.pop()
-            if event.start != start:
-                start = event.start
-                start_index += 1
-            event.start_index = start_index
+            if event.start != current_endpoint:
+                current_endpoint = event.start
+                current_endpoint_id += 1
+            event.start_id = current_endpoint_id
             self.process_event(event, result, sweep_line)
         return result
 
@@ -235,14 +237,14 @@ class CompleteIntersection(Operation):
             same_start_events = list(same_start_events)
             if not (any(event.primary.wholly_in_complete_intersection
                         for event in same_start_events)
-                    or all_equal(event.from_first
+                    or all_equal(event.from_first_operand
                                  for event in same_start_events)):
                 points.append(start)
         segments = endpoints_to_segments(
                 [to_endpoints(event)
                  for event in events
                  if (event.is_left
-                     and event.from_first
+                     and event.from_first_operand
                      and event.is_common_polyline_component)],
                 context
         )
@@ -253,7 +255,8 @@ class CompleteIntersection(Operation):
 
     def from_shaped_result(self, event: LeftEvent) -> bool:
         return (event.inside
-                or not event.from_first and event.is_common_region_boundary)
+                or (not event.from_first_operand
+                    and event.is_common_region_boundary))
 
     def sweep(self) -> List[Event]:
         self.fill_queue()
@@ -263,17 +266,17 @@ class CompleteIntersection(Operation):
         min_max_x = min(to_polygons_x_max(self.first.polygons),
                         to_polygons_x_max(self.second.polygons))
         event = events_queue.pop()
-        start = event.start
-        start_index = event.start_index = 0
+        current_endpoint = event.start
+        current_endpoint_id = event.start_id = 0
         self.process_event(event, result, sweep_line)
         while events_queue:
             event = events_queue.pop()
             if min_max_x < event.start.x:
                 break
-            if event.start != start:
-                start = event.start
-                start_index += 1
-            event.start_index = start_index
+            if event.start != current_endpoint:
+                current_endpoint = event.start
+                current_endpoint_id += 1
+            event.start_id = current_endpoint_id
             self.process_event(event, result, sweep_line)
         return result
 
@@ -296,7 +299,7 @@ class Difference(Operation):
 
     def from_shaped_result(self, event: LeftEvent) -> bool:
         return (event.outside
-                if event.from_first
+                if event.from_first_operand
                 else event.inside or event.is_common_polyline_component)
 
     def sweep(self) -> List[Event]:
@@ -306,17 +309,17 @@ class Difference(Operation):
         sweep_line = SweepLine(self.context)
         first_x_max = to_polygons_x_max(self.first.polygons)
         event = events_queue.pop()
-        start = event.start
-        start_index = event.start_index = 0
+        current_endpoint = event.start
+        current_endpoint_id = event.start_id = 0
         self.process_event(event, result, sweep_line)
         while events_queue:
             event = events_queue.pop()
             if first_x_max < event.start.x:
                 break
-            if event.start != start:
-                start = event.start
-                start_index += 1
-            event.start_index = start_index
+            if event.start != current_endpoint:
+                current_endpoint = event.start
+                current_endpoint_id += 1
+            event.start_id = current_endpoint_id
             self.process_event(event, result, sweep_line)
         return result
 
@@ -344,7 +347,8 @@ class Intersection(Operation):
 
     def from_shaped_result(self, event: LeftEvent) -> bool:
         return (event.inside
-                or not event.from_first and event.is_common_region_boundary)
+                or (not event.from_first_operand
+                    and event.is_common_region_boundary))
 
     def sweep(self) -> List[Event]:
         self.fill_queue()
@@ -354,17 +358,17 @@ class Intersection(Operation):
         min_max_x = min(to_polygons_x_max(self.first.polygons),
                         to_polygons_x_max(self.second.polygons))
         event = events_queue.pop()
-        start = event.start
-        start_index = event.start_index = 0
+        current_endpoint = event.start
+        current_endpoint_id = event.start_id = 0
         self.process_event(event, result, sweep_line)
         while events_queue:
             event = events_queue.pop()
             if min_max_x < event.start.x:
                 break
-            if event.start != start:
-                start = event.start
-                start_index += 1
-            event.start_index = start_index
+            if event.start != current_endpoint:
+                current_endpoint = event.start
+                current_endpoint_id += 1
+            event.start_id = current_endpoint_id
             self.process_event(event, result, sweep_line)
         return result
 
@@ -403,7 +407,7 @@ class Union(Operation):
 
     def from_shaped_result(self, event: LeftEvent) -> bool:
         return (event.outside
-                or not event.from_first and event.is_common_region_boundary)
+                or not event.from_first_operand and event.is_common_region_boundary)
 
 
 def _compute_relations(event: LeftEvent,
@@ -437,44 +441,50 @@ def _compute_relations(event: LeftEvent,
 
 
 def _to_contour_events(event: LeftEvent,
-                       events: Sequence[LeftEvent],
+                       events: Sequence[Event],
                        connectivity: Sequence[int],
-                       processed: Sequence[bool],
-                       visited_positions: List[int]) -> List[Event]:
+                       are_events_processed: Sequence[bool],
+                       visited_endpoints_positions: List[int]) -> List[Event]:
+    assert event.is_left
     result = [event]
-    visited_positions[event.start_index] = 0
-    opposite_position = event.right.position
+    visited_endpoints_positions[event.start_id] = 0
+    opposite_event_id = event.right.id
     contour_start = event.start
     cursor = event
+    visited_endpoints_ids = [event.start_id]
     while cursor.end != contour_start:
-        if visited_positions[cursor.end_index] == UNDEFINED_INDEX:
-            visited_positions[cursor.end_index] = len(result)
+        previous_endpoint_position = visited_endpoints_positions[cursor.end_id]
+        if previous_endpoint_position == UNDEFINED_INDEX:
+            visited_endpoints_positions[cursor.end_id] = len(result)
         else:
             # vertices loop found, i.e. contour has self-intersection
-            previous_vertex_position = visited_positions[cursor.end_index]
-            assert previous_vertex_position != 0
-            for event in result[previous_vertex_position:]:
-                visited_positions[event.end_index] = UNDEFINED_INDEX
-            del result[previous_vertex_position:]
-        position = _to_next_position(opposite_position, processed,
+            assert previous_endpoint_position != 0
+            for loop_event in result[previous_endpoint_position:]:
+                visited_endpoints_positions[
+                    loop_event.end_id
+                ] = UNDEFINED_INDEX
+            del result[previous_endpoint_position:]
+        visited_endpoints_ids.append(cursor.end_id)
+        event_id = _to_next_event_id(opposite_event_id, are_events_processed,
                                      connectivity)
-        if position is None:
+        if event_id == UNDEFINED_INDEX:
             break
-        cursor = events[position]
-        opposite_position = cursor.opposite.position
+        cursor = events[event_id]
+        opposite_event_id = cursor.opposite.id
         result.append(cursor)
-    visited_positions[result[0].start_index] = UNDEFINED_INDEX
-    for event in result:
-        visited_positions[event.end_index] = UNDEFINED_INDEX
-    assert all(position == UNDEFINED_INDEX for position in visited_positions)
+    for event_id in visited_endpoints_ids:
+        visited_endpoints_positions[event_id] = UNDEFINED_INDEX
+    assert all(position == UNDEFINED_INDEX
+               for position in visited_endpoints_positions)
     return result
 
 
 def _process_contour_events(contour_events: Iterable[Event],
                             contour_id: int,
-                            processed: List[bool]) -> None:
+                            are_events_processed: List[bool]) -> None:
     for event in contour_events:
-        processed[event.position] = processed[event.opposite.position] = True
+        are_events_processed[event.id] = True
+        are_events_processed[event.opposite.id] = True
         if event.is_left:
             event.from_in_to_out = False
             event.contour_id = contour_id
@@ -483,13 +493,13 @@ def _process_contour_events(contour_events: Iterable[Event],
             event.opposite.contour_id = contour_id
 
 
-def _to_next_position(position: int,
-                      processed: Sequence[bool],
-                      connectivity: Sequence[int]) -> Optional[int]:
-    candidate = position
+def _to_next_event_id(event_id: int,
+                      are_events_processed: Sequence[bool],
+                      connectivity: Sequence[int]) -> int:
+    candidate = event_id
     while True:
         candidate = connectivity[candidate]
-        if not processed[candidate]:
+        if not are_events_processed[candidate]:
             return candidate
-        elif candidate == position:
-            return None
+        elif candidate == event_id:
+            return UNDEFINED_INDEX
